/*
    4/4/03
    Nathan Seidle
    nathan.seidle@colorado.edu
    Copyright Spark Fun Electronics 2003
    
    Protected under the following license:
    If you find this code helpful, shoot me an email and let me know! I'd love to
    hear about your project. You are welcome to modify and use this code.
    If you use this code and make lots of money, buy me a beer for royalties.
    
    
    Joystick Control of Small Robot
    
    Need to read the location of two joysticks and pass that information
    down a 250 ft. tether to the robot. The PIC will read the joysticks,
    encode the information, and serially send the info down an RS485 industrial
    serial line.
    
    This is the Robot Receive and Motor Control portion of the system.
    
    The data stream consists of:
    Byte 0 - 0xD4 - Start byte

    Byte 1 - 0b.xxxx.xxxx - X Position of Joystick 1 High Byte
    Byte 2 - 0b.xxxx.xxxx - X Position of Joystick 1 Low Byte
    Byte 3 - 0b.xxxx.xxxx - Y Position of Joystick 1 High Byte
    Byte 4 - 0b.xxxx.xxxx - Y Position of Joystick 1 Low Byte

    Byte 5 - 0b.xxxx.xxxx - X Position of Joystick 2 High Byte
    Byte 6 - 0b.xxxx.xxxx - X Position of Joystick 2 Low Byte
    Byte 7 - 0b.xxxx.xxxx - Y Position of Joystick 2 High Byte
    Byte 8 - 0b.xxxx.xxxx - Y Position of Joystick 2 Low Byte

    Byte 9 - 0b.xxxx.xxxx - Dimmer Position High Byte
    Byte 10 - 0b.xxxx.xxxx - Dimmer Position Low Byte

    Byte 11 - 0b.1110.1101 0x4D - Stop Byte     
    
    4-15-03 Tax Day - Received stepper motor and MBC15161 Anaheim Automation controller. 
    Got motor to move! Best setup is a 300us high pulse followed by 300us low pulse.
    Pins 2,4,6 are tied low. 1 is pulsed.
    Motor Side : 1-Orange 2-Green 3-Blue 4-Brown
    
    Forward works great - Reverse is horribly choppy. Clock LED looks odd.
    
    4-16-03 The receiver now works like a charm! The code looks like spagetti, but by not using
    loops, the code runs 3-4 times faster. I can achieve clock rates higher than the max of the 
    motor - just barely. The transmission specification must be changed. The receiver cannot afford
    to recalculate the appropriate timer values. Total tranmission time is less!
    Bytes must now be transmitted as follows:

    Byte 0 - 0xD4 - Start byte

    Byte 1 - 0b.xxxx.xxxx - Motor 1 Pulse Time
    Byte 2 - 0b.xxxx.xxxx - Motor 2 Pulse Time
    Byte 3 - 0b.xxxx.xxxx - Motor 3 Pulse Time
    Byte 4 - 0b.xxxx.xxxx - Motor 4 Pulse Time
    Byte 5 - 0b.xxxx.xxxx - LED Pulse Time
    Byte 6 - 0b.xxxx.xxxx - Direction byte - Direction for Motor 0b.0005.4321
    
    Byte 7 - 0b.1110.1101 0x4D - Stop Byte  
    
    4-17-03 The transmitter has been touched up. The receiver has been stream lined. Everything works!
    Port A is checked for reverse jumpers. Install a jumper on PORTA0 to high to reverse the control
    stick.
    
    As it stands now: - Updated 5-4-03
    Pin 1  Vpp - Reset Tied High
    Pin 2  RA0 - Motor 1 Direction
    Pin 3  RA1 - Motor 2 Direction
    Pin 4  RA2 - Motor 3 Direction
    Pin 5  RA3 - Motor 4 Direction
    Pin 6  RA4 - NC
    Pin 7  RA5 - NC
    Pin 8  Vss - GND
    Pin 9  OCS1 - Xtal
    Pin 10 OSC2 - Xtal       
    Pin 11 RC0 - DC Motor 4
    Pin 12 RC1 - DC Motor 3
    Pin 13 RC2 - DC Motor 2
    Pin 14 RC3 - DC Motor 1

    Pin 15 RC4 - NC
    Pin 16 RC5 - Serial Out
    Pin 17 RC6 - NC
    Pin 18 RC7 - Data Receive from RS-485 or Wireless line
    Pin 19 Vss - GND
    Pin 20 Vdd - Vdd
    Pin 21 RB0 - Motor 1 Pulse
    Pin 22 RB1 - Motor 2 Pulse
    Pin 23 RB2 - Motor 3 Pulse
    Pin 24 RB3 - Motor 4 Pulse
    Pin 25 RB4 - Motor 5 Pulse
    Pin 26 RB5 - NC
    Pin 27 RB6 - NC - Programming RB6
    Pin 28 RB7 - NC - Programming RB7

    4-26-03 3:55am Steering Motor Settings: 1off 2on 3off 4off - also all off (slower)
    
*/

#include "d:\Pics\c\16F873.h"  // device dependent interrupt definitions
#include "d:\Pics\c\int16CXX.H"

#pragma config |= 0x3F32 // LVP Off, WDTE Off, HS Oscillator

#pragma origin 4

//Definitions
#define TRUE        1
#define FALSE       0
#define TRANSMIT_SIZE   8
#define TOTAL_MOTORS    6

//Global Variables
bit print_it;
bit start_data;

uns8 data_spot;
uns8 data_array[30];

uns8 time_left[TOTAL_MOTORS];
uns8 load_value[TOTAL_MOTORS];
uns8 direction;

//Interrupt Vectors
interrupt serverX( void)
{
    int_save_registers
    char sv_FSR = FSR;  // save FSR if required

    if(RCIF) //UART Recieve Interrupt
    {
        
        data_array[data_spot] = RCREG;

        if(RCREG == 0xD4) start_data = TRUE;
        //if(RCREG == 0x4D)
            if(data_spot == (TRANSMIT_SIZE-1) )
                print_it = TRUE;

        if(start_data) data_spot++;

            
        //No clearing RCIF, must clear RCREG        
        RCREG = 0;
    }

    FSR = sv_FSR;               // restore FSR if saved
    int_restore_registers 
}

#include "d:\Pics\code\Delay.c"   // Delays. What do you expect?
//#define Serial_Out  RC5
#include "d:\Pics\code\Stdio.c"   // Basic Serial IO

//Function Definitions
void init_values(void);

void main(void)
{

    PORTA = 0b.0000.0000; 
    TRISA = 0b.0000.0000;  //0 = Output, 1 = Input
                            
    PORTB = 0b.0000.0000; 
    TRISB = 0b.0000.0000;

    PORTC = 0b.0000.0000; 
    TRISC = 0b.0000.0000;  //0 = Output, 1 = Input

    //rs_out('k');
    
    //delay_s();
    
    //Initialize variables
    init_values();
    int k, i;
    
    //Turn Port A to Digital Outputs
    ADCON1 = 0b.0000.0111;

    //Initialize UART Interrupts
    //====================================
    //SPBRG = 12; //4MHz 4800
    SPBRG = 64; //20MHz 4800
    //SPBRG = 32; //20MHz 9600 = 31.54

    WREN = 1;
    SYNC = 0;
    SPEN = 1;

    GIE = 1;
    PEIE = 1;
    RCIE = 1;

    CREN = 1;
    //=====================================
    
    uns8 ctrl_byte = 0b.0000.0000;

    //Setup Camera Servos
    uns8 camera_byte = 0b.0000.0000;
    
    //Setup one steering motor opposite direction
    direction = 0b.0000.1111;
    
    //Main Loop
    while(1)
    {
        //Decrement timers
        time_left[0]--;
        time_left[1]--;
        time_left[2]--;
        time_left[3]--;
        time_left[4]--;
        time_left[5]--;

        //Flip bits in control byte - pulses

        //Drive Motors
        if (time_left[0] == 0)
        {
            k = load_value[0];
            time_left[0] = k;
            if (time_left[0] < 250) ctrl_byte = ctrl_byte ^ 0b.0000.0011;
        }

        //Steering Motors
        if (time_left[1] == 0)
        {
            k = load_value[1];
            time_left[1] = k;
            if (time_left[1] < 250)
            {
                //Pulse both steering motors VERY SLOWLY
                ctrl_byte = ctrl_byte ^ 0b.0000.1100;
            }
        }

        //Camera Servos
        //(RC3  1in) (RC2 2in) (RC1 3in) (RC0 4in)
        if (time_left[2] == 0)
        {
            k = load_value[2];
            time_left[2] = k;

            //Pulse current direction pin
            if (time_left[2] < 250)
            {
                //If we are rotating on 2in, pulse RC2
                if (direction.4 == 0) camera_byte = camera_byte ^ 0b.0000.1000; //One goes high, one low
                if (direction.4 == 1) camera_byte = camera_byte ^ 0b.0000.0100; //One goes high, one low
            }
        }
        if (time_left[3] == 0)
        {
            k = load_value[3];
            time_left[3] = k;

            //Pulse current direction pin
            if (time_left[3] < 250)
            {
                //If we are rotating on 3in, pulse RC1
                if (direction.5 == 1) camera_byte = camera_byte ^ 0b.0000.0010; //One goes high, one low
                if (direction.5 == 0) camera_byte = camera_byte ^ 0b.0000.0001; //One goes high, one low
            }
        }
        
        
        //Other - LEDs
        if (time_left[4] == 0)
        {
            k = load_value[4];
            time_left[4] = k;
            if (time_left[4] < 250) ctrl_byte = ctrl_byte ^ 0b.0001.0000;
        }

        if (time_left[5] == 0) 
        {
            k = load_value[5];
            time_left[5] = k;
            if (time_left[5] < 250) ctrl_byte = ctrl_byte ^ 0b.0010.0000;
        }
        
        //Set directions
        PORTA = direction;

        //Write control byte out to port
        PORTB = ctrl_byte;

        //Write camera byte out to port
        PORTC = camera_byte;

        //if(de_bug) load_value[1] = 130;
    
        //See if we have new orders from the controller
        if (print_it == TRUE) 
        {
            GIE = 0;

            print_it = FALSE;
            start_data = FALSE;
            data_spot = 0;
            
            load_value[0] = data_array[1];
            load_value[1] = data_array[2];
            load_value[2] = data_array[3];
            load_value[3] = data_array[4];
            load_value[4] = data_array[5];
            direction = data_array[6];

            GIE = 1;
            CREN = 0;
            CREN = 1;
        }
        
        //delay_ms(10); //Debugging delay for listening to motors
        
        //Serial out for debugging info
        /*for(i = 0 ; i < 5 ; i++)
        {
            printf(strh, load_value[i]);
            rs_out(' ');
        }
        rs_out('-');
        printf(strb, direction);
        rs_out(' ');
        rs_out('C');
        rs_out('-');
        printf(strb, camera_byte);
        rs_out(' ');
        rs_out('M');
        rs_out('-');
        printf(strb, ctrl_byte);
        rs_out('\n');
        rs_out('\r');*/
    
        
    }
        
}// End Main

//Zeros out all the variables
//Inits the major arrays
void init_values(void)
{
    int j;
    for (j = 0 ; j < 10 ; j++)
        data_array[j] = 0;
    
    print_it = FALSE;
    start_data = FALSE;
    data_spot = 0;

    load_value[0] = 248;
    load_value[1] = 230;
    load_value[2] = 255;
    load_value[3] = 255;
    load_value[4] = 255;
    load_value[5] = 255;
    direction = 0b.0000.1111;
}    